const db = require("../../infra/database/db");
require("dotenv").config({ path: __dirname + "/./../../../.env" });
const jwt = require("jsonwebtoken");

module.exports.isAuthenticatedUser = (req, res, next) => {


  try {
    let token;

    if (req.headers.authorization) {
      const authHeader = req.headers.authorization;
      if (authHeader.startsWith('Bearer ')) {
        token = authHeader.split(' ')[1];
      } else {
        token = authHeader;
      }
    } else if (req.query.auth) {
      token = req.query.auth;
    }

    if (!token) {
      return res.status(401).json({ error: "Token not provided" });
    }

    const decoded = jwt.decode(token);
    const authValue = decoded ? decoded.auth : null;

    if (!authValue) {
      return res.status(401).json({ error: "Invalid token" });
    }

    db.query(
      `SELECT * FROM user WHERE auth=? AND (role=? OR role=?)`,
      [authValue, "USER", "ADMIN"],
      async (error, usuarios) => {
        if (error) {
          console.error("Database Error: ", error);
          return res.status(500).json({ error: "Internal Server Error" });
        }

        if (usuarios && usuarios.length > 0) {
          console.log("User found: ", usuarios[0]);
          next();
        } else {
          res.status(401).json({ error: "Rota bloqueada" });
        }
      }
    );
  } catch (err) {
    console.error("Authorization Error: ", err);
    res.status(401).json({ error: "Não autorizado!" });
  }
}

module.exports.authorizeRole = (req, res, next) => {
  return (req, res, next) => {
    if (req.user.role) {
      return res.status(403).json({ error: 'Acesso negado' });
    }
    next();
  };
}


module.exports.isAuthenticatedAdmin = (req, res, next) => {
  try {
    let token;

    if (req.headers.authorization) {
      const authHeader = req.headers.authorization;
      if (authHeader.startsWith('Bearer ')) {
        token = authHeader.split(' ')[1];
      } else {
        token = authHeader;
      }
    } else if (req.query.auth) {
      token = req.query.auth;
    }

    if (!token) {
      return res.status(401).json({ error: "Token not provided" });
    }

    const decoded = jwt.decode(token);
    const authValue = decoded ? decoded.auth : null;

    if (!authValue) {
      return res.status(401).json({ error: "Invalid token" });
    }

    db.query(
      `SELECT * FROM user WHERE auth=? AND role=?`,
      [authValue, "ADMIN"],
      async (error, usuarios) => {
        if (error) {
          console.error("Database Error: ", error);
          return res.status(500).json({ error: "Internal Server Error" });
        }

        if (usuarios && usuarios.length > 0) {
          console.log("User found: ", usuarios[0]);
          next();
        } else {
          res.status(401).json({ error: "Rota bloqueada" });
        }
      }
    );
  } catch (err) {
    console.error("Authorization Error: ", err);
    res.status(401).json({ error: "Não autorizado!" });
  }
}

module.exports.isAuthenticatedDev = (req, res, next) => {
  try {
    const token = req.headers.authorization
      ? jwt.decode(req.headers.authorization).auth
      : req.query.auth
        ? req.query.auth
        : "";
    db.query(
      `SELECT * FROM usuarios WHERE auth=? and perfil=?`,
      [token, "DEV"],
      async (error, usuarios) => {
        if (error) return reject(new Error(error));
        if (usuarios[0]) {
          next();
        } else {
          res.status(401).json({ error: "Rota bloqueada" });
        }
      }
    );
  } catch {
    var erro = new Error("Não autorizado!");
    erro.status = 401;
    return next(erro);
  }
};


module.exports.isAuthenticated = (req, res, next) => {
  try {
    const token = req.headers.authorization
      ? jwt.decode(req.headers.authorization).auth
      : req.query.auth
        ? req.query.auth
        : "";
    db.query(
      `SELECT * FROM usuarios WHERE auth=?`,
      [token],
      async (error, usuarios) => {
        if (error) return reject(new Error(error));
        if (usuarios[0]) {
          next();
        } else {
          res.status(401).json({ error: "Rota bloqueada" });
        }
      }
    );
  } catch {
    var erro = new Error("Não autorizado!");
    erro.status = 401;
    return next(erro);
  }
};
