const db = require("../infra/database/db");
const Admin = require("../infra/repository/admin.repository");
const admin = new Admin();
const jwt = require("jsonwebtoken");

class adminController {

  login() {
    return function (req, res) {
      const data = req.body;


      admin
        .login(data)
        .then((response) => {
          if (response.error)
            return res.status(response.code).send({ error: response.error });
          else return res.status(200).json(response);
        })
        .catch((error) => {
          console.log(error);
          return res
            .status(400)
            .send({ error: `Não foi possivel realizar o login` });
        });
    };
  }

  register() {
    return function (req, res) {


      const data = req.body;


      admin
        .register(data)
        .then((response) => {
          if (response.error)
            return res.status(response.code).send({ error: response.error });
          else return res.status(200).json(response);
        })
        .catch((error) => {
          console.log(error);

          return res
            .status(400)
            .send({ error: `Não foi possível realizar o cadastro!` });
        });
    };
  }

  recover() {
    return function (req, res) {
      const  data  = req.body;
      admin
        .recover(data)
        .then((response) => {
          if (response.error)
            return res.status(response.code).send({ error: response.error });
          else return res.status(200).json(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }


 // Função que conta os tipos de usuários registrados
 countAllUserTypes() {
  return new Promise((resolve, reject) => {
      
    if (!db) return reject({ error: "Banco de dados não configurado." });
      

      db.query(
          `SELECT  
          (
              SELECT COUNT(*) FROM vendedores
          ) AS vendedores,
          (
              SELECT COUNT(*) FROM user WHERE user.status like "aprovado"
          ) AS aprovados,
          (
              SELECT COUNT(*) FROM user WHERE user.status like "recusado"
          ) AS recusados,
          (
              SELECT COUNT(*) FROM user WHERE user.status like "pendente"
          ) AS pendentes
          FROM 
          user
          WHERE 
          user.role != 'ADMIN'
          GROUP BY vendedores`,
          [],
          (error, result) => {
              if (error) {
                  console.log(error);
                  return reject({ error: "Erro ao consultar a quantidade total de usuários." });
              }

              console.log('Controller foi executado e está enviando os seguintes dados:', result);
              resolve(result); // Resolve a Promise com o resultado
          }
      );
  });
}

list() {
  return new Promise(async (resolve, reject) => {
    try {
      db.query(`SELECT  
              (
              SELECT COUNT(*) FROM vendedores
              ) AS vendedores,

              (
              SELECT COUNT(*) FROM funcionarios WHERE funcionarios.status like "aprovado"
              ) AS func_aprovados,
              (
               SELECT COUNT(*) FROM funcionarios WHERE funcionarios.status like "pendente"
              ) AS func_pendentes,

              (
              SELECT COUNT(*) FROM user WHERE user.role != "ADMIN" AND user.status like "aprovado"
              ) AS aprovados,
              (
              SELECT COUNT(*) FROM user WHERE user.role != "ADMIN" AND user.status like "pendente"
              ) AS pendentes,
              (
              SELECT COUNT(*) FROM produtos
              ) AS produtos,
              (
               SELECT 
                  COUNT(*) AS users_pedido_hj
              FROM user u
              WHERE 
                  u.status = 'aprovado' AND 
                  u.role != 'ADMIN' AND 
                  u.dias_pedido LIKE CONCAT('%', 
                      CASE DAYOFWEEK(NOW())
                          WHEN 1 THEN 'Domingo'
                          WHEN 2 THEN 'Segunda'
                          WHEN 3 THEN 'Terça'
                          WHEN 4 THEN 'Quarta'
                          WHEN 5 THEN 'Quinta'
                          WHEN 6 THEN 'Sexta'
                          WHEN 7 THEN 'Sábado'
                      END, 
                  '%')
              ) AS users_pedido_hj,
              (
                        SELECT 
              COUNT(*) AS users_pedido_pendentes
          FROM user u
          WHERE 
              u.status = 'aprovado' AND 
              u.role != 'ADMIN' AND 
              u.id NOT in (
                 SELECT p.id_user 
                  FROM pedidos p 
                  WHERE DATE(p.data) = CURDATE()
              ) AND
              u.dias_pedido LIKE CONCAT('%', 
                  CASE DAYOFWEEK(NOW())
                      WHEN 1 THEN 'Domingo'
                      WHEN 2 THEN 'Segunda'
                      WHEN 3 THEN 'Terça'
                      WHEN 4 THEN 'Quarta'
                      WHEN 5 THEN 'Quinta'
                      WHEN 6 THEN 'Sexta'
                      WHEN 7 THEN 'Sábado'
                  END, 
              '%')
              ) AS users_pedido_pendentes,
              (
                SELECT COUNT(*) AS pedidos_concluidos
                FROM (
                  SELECT p.id_user, p.data
                  FROM pedidos p
                  JOIN user u ON u.id = p.id_user
                  WHERE 
                    p.concluido = true
                    AND MONTH(p.data) = MONTH(CURDATE())
                    AND YEAR(p.data) = YEAR(CURDATE())
                    AND u.role != 'ADMIN'
                  GROUP BY p.id_user, p.data
                ) AS pedidos_concluidos
              ) AS pedidos_concluidos
              FROM 
              user
              WHERE 
              user.role != 'ADMIN'
              GROUP BY vendedores`, [],
        async (error, resposta) => {

          if (error) {

            console.log(error);
            return reject(new Error(error));
          }

          
          return resolve(resposta);
        });
    } catch (error) {
      return reject(new Error(error));
    }
  });
}


// Função que configura o listener do socket
setupSocketListenerCount() {

  const teste = "LUISAO"
  

    // const { id_user } = data;

    // if (!id_user) {
    //     socket.emit('countAllUserTypesResponse', { error: "ID do usuário é obrigatório." });
    //     return;
    // }
    

    

    return this.countAllUserTypes();
    

    // this.countAllUserTypes( (error, response) => {
    //     if (error) {
    //         console.log({ error: error });
            
    //     } else {
    //       console.log("TOMA! ", response);
          
    //     }
    // });

}

  logout() {
    return function (req, res) {
      const auth = req.query.auth ? req.query.auth : "";
      admin
        .logout(auth)
        .then((response) => {
          if (response.error)
            return res.status(response.code).send({ error: response.error });
          else return res.status(200).json(response);
        })
        .catch((error) => {
          return res
            .status(400)
            .send({ error: `Não foi possivel realizar o login` });
        });
    };
  }


  listUserPendente() {
    return function (req, res) {
      
      const data_criado = req.query.data_criado

      admin
        .listUserPendente(data_criado)
        .then((response) => {
          if (response.error)
            res.status(response.code).send({ error: response.error });
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }


  listUserAprovado() {
    return function (req, res) {
      const { id } = req.params;
      const data_criado = req.query.data_criado

      admin
        .listUserAprovado(data_criado)
        .then((response) => {
          if (response.error)
            res.status(response.code).send({ error: response.error });
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  listEdit() {
    return function (req, res) {
      const id_user = req.query.id_user

      admin
        .listEdit(id_user)
        .then((response) => {
          if (response.error)
            res.status(response.code).send({ error: response.error });
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }


  listUserRecusado() {
    return function (req, res) {
      const { id } = req.params;
      const data_criado = req.query.data_criado


      admin
        .listUserRecusado(data_criado)
        .then((response) => {
          if (response.error)
            res.status(response.code).send({ error: response.error });
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }


countAllUserTypes() {
  return function (req, res) {


    admin
      .countAllUserTypes()
      .then((response) => {
        if (response.error)
          res.status(response.code).send({ error: response.error });
        else return res.status(200).send(response);
      })
      .catch((error) => {
        throw new Error(error);
      });
  };
}

   /**
* Lista pedido que o usuario concluído
* @returns lista com todos os dados de pedidos conluido 
*/
listUserPedidosConluidoSocket(data_inicio, data_fim) {
  return new Promise(async (resolve, reject) => {
    // const query = data_concluido ? `pedidos.data = ?` : `pedidos.data <= ?`
    // const data_formatada = data_concluido ? data_concluido : new Date();

  
    try {
      db.query(`SELECT pedidos.id, pedidos.id_user, user.razao_social, user.documento, SUM(pedidos.quantidade) AS quantidade_total, DATE_FORMAT(pedidos.data, '%d/%m/%Y') AS data, pedidos.hora
                  FROM user, pedidos
                  WHERE
                  pedidos.id_user = user.id AND
                  pedidos.concluido = true AND
                  pedidos.data >  ? AND
                  pedidos.data < ?
                  GROUP BY user.id, pedidos.data
                  ORDER BY pedidos.data DESC, pedidos.hora DESC `,
        [data_inicio, data_fim],

        (error, pedidos) => {
          if (error) return reject(new Error(error));
          return resolve(pedidos);
        }
      );
    } catch (error) {
      return reject(new Error(error));
    }
  });
}


listUserPedidosConluido() {
  return function (req, res) {
      const data_concluido = req.query.data_concluido;

    admin
      .listUserPedidosConluido(data_concluido)
      .then((response) => {
        if (response.error)
          res.status(response.code).send({ error: response.error });
        else return res.status(200).send(response);
      })
      .catch((error) => {
        throw new Error(error);
      });
  };
}

listPedidosPorEndereco() {
  return function (req, res) {
      const endereco = req.query.endereco;

    admin
      .listPedidosPorEndereco(endereco)
      .then((response) => {
        if (response.error)
          res.status(response.code).send({ error: response.error });
        else return res.status(200).send(response);
      })
      .catch((error) => {
        throw new Error(error);
      });
  };
}

listPedidosNaoEfetuados() {
  return function (req, res) {

    admin
      .listPedidosNaoEfetuados()
      .then((response) => {
        if (response.error)
          res.status(response.code).send({ error: response.error });
        else return res.status(200).send(response);
      })
      .catch((error) => {
        throw new Error(error);
      });
  };
}

  listUserAprovadoTopAS() {
    return function (req, res) {
      const { id } = req.params;

      admin
        .listUserAprovadoTopAS(id)
        .then((response) => {
          if (response.error)
            res.status(response.code).send({ error: response.error });
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }


  listUserAprovadoAtacado() {
    return function (req, res) {
      const { id } = req.params;

      admin
        .listUserAprovadoAtacado(id)
        .then((response) => {
          if (response.error)
            res.status(response.code).send({ error: response.error });
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  /**
* Lista todos os usuarios aprovados
* @returns mensagem referente ao resultado obtido
*/
  listUserAprovadoDistribuidor() {
    return function (req, res) {
      const { id } = req.params;

      admin
        .listUserAprovadoDistribuidor(id)
        .then((response) => {
          if (response.error)
            res.status(response.code).send({ error: response.error });
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  /**
  * Lista todos os usuarios aprovados
  * @returns mensagem referente ao resultado obtido
  */
  listHistoricoPedidoByUser() {
    return function (req, res) {
      const id_user = req.query.id_user;
      const data_inicio = req.query.data_inicio;
      const data_fim = req.query.data_fim;



      admin
        .listHistoricoPedidoByUser(id_user, data_inicio, data_fim)
        .then((response) => {
          if (response.error)
            res.status(response.code).send({ error: response.error });
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  listAllUserDiaPedido() {
    return function (req, res) {
      const data = req.query.data

      admin
        .listAllUserDiaPedido(data)
        .then((response) => {
          if (response.error)
            res.status(response.code).send({ error: response.error });
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }




  listAllUser() {
    return function (req, res) {
      const { id } = req.params;

      admin
        .listAllUser(id)
        .then((response) => {
          if (response.error)
            res.status(response.code).send({ error: response.error });
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }


  listUserHistorico() {
    return function (req, res) {
      const { id } = req.params;

      admin
        .listUserHistorico(id)
        .then((response) => {
          if (response.error)
            res.status(response.code).send({ error: response.error });
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  
  listTopVendedores() {
    return function (req, res) {

      admin
        .listTopVendedores()
        .then((response) => {
          if (response.error)
            res.status(response.code).send({ error: response.error });
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  listTopCidades() {
    return function (req, res) {

      admin
        .listTopCidades()
        .then((response) => {
          if (response.error)
            res.status(response.code).send({ error: response.error });
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  /**
   * Delete um usuário
   * @returns mensagem referente ao resultado obtido
   */
  delete() {
    return function (req, res) {
      const { id } = req.params;

      admin
        .delete(id)
        .then((response) => {
          if (response.error) res.status(response.code).send(response);
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  update() {
    return function (req, res) {
      const  data  = req.body;

      admin
        .update(data)
        .then((response) => {
          if (response.error) res.status(400).send(response);
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

    /**
   * Atualiza os dados de um usuário
   * @returns mensagem referente ao resultado obtido
   */
    updateFullUser() {
      return function (req, res) {
        const data = req.body;
  
        admin
          .updateFullUser(data)
          .then((response) => {
            if (response.error) res.status(400).send(response);
            else return res.status(200).send(response);
          })
          .catch((error) => {
            throw new Error(error);
          });
      };
    }

    desvincularVendedor() {
      return function (req, res) {
        const  data  = req.body;
  
        admin
          .desvincularVendedor(data)
          .then((response) => {
            if (response.error) res.status(response.code).send(response);
            else return res.status(200).send(response);
          })
          .catch((error) => {
            throw new Error(error);
          });
      };
    }

  /**
   * Criptografa um valor
   * @returns o valor criptografado
   */
  encrypt() {
    return function (req, res) {
      const { data } = req.body;

      admin
        .encrypt(data)
        .then((response) => {
          if (response.error)
            return res.status(response.code).send({ error: response.error });
          else return res.status(200).json(response);
        })
        .catch((error) => {
          return res
            .status(400)
            .send({ error: `Não foi possível criptografar!` });
        });
    };
  }




  /**
 * Atualiza os dados de um usuário
 * @returns mensagem referente ao resultado obtido
 */
  aprovarUsuario() {
    return function (req, res) {
      const data = req.body;

      admin
        .aprovarUsuario(data)
        .then((response) => {
          if (response.error) res.status(400).send(response);
          
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  setStatusFunc() {
    return function (req, res) {
      const data = req.body;

      admin
        .setStatusFunc(data)
        .then((response) => {
          if (response.error) res.status(400).send(response);
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  aprovarFuncionario() {
    return function (req, res) {
      const data = req.body;

      admin
        .aprovarFuncionario(data)
        .then((response) => {
          if (response.error) res.status(400).send(response);
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  
  recusarFuncionario() {
    return function (req, res) {
      const data = req.body;

      admin
        .recusarFuncionario(data)
        .then((response) => {
          if (response.error) res.status(400).send(response);
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  /**
* Atualiza os dados de um usuário
* @returns mensagem referente ao resultado obtido
*/
  RecusarUsuario() {
    return function (req, res) {
      const data = req.body;

      admin
        .RecusarUsuario(data)
        .then((response) => {
          if (response.error) res.status(400).send(response);
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  /**
* Atualiza os dados de um usuário
* @returns mensagem referente ao resultado obtido
*/
  edit() {
    return function (req, res) {
      const data = req.body;
      const id_user = req.query.id_user

      console.log(data)

      admin
        .edit(data, id_user)
        .then((response) => {
          if (response.error) res.status(400).send(response);
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  updateUltra() {
    return function (req, res) {
        const  data  = req.body;
        
        admin.updateUltra(data).then((response) => {
            if (response.error) res.status(400).send(response);
            else return res.status(200).send(response);
        }).catch((error) => {
            throw new Error(error);
        });
    }
}

listUltra() {
  return function (req, res) {
      
      admin.listUltra().then((response) => {
          if (response.error) res.status(response.status).send(response);
          else return res.status(200).send(response);
      }).catch((error) => {
          throw new Error(error);
      });
  }
}
}

module.exports = adminController;
