const Produtos = require("../infra/repository/produtos.repository");
const produtos = new Produtos();

class produtosController {


    /**
     * Registra um usuário
     * @returns mensagem referente ao resultado obtido
     */
    register() {
        return function (req, res) {
            const data = req.body;
            const file = req.file;

            produtos.register(data, file).then((response) => {
                if (response.error) return res.status(response.code).send({ error: response.error });
                else return res.status(200).json(response);
            }).catch((error) => {
                return res.status(400).send({ error: `Não foi possível realizar o cadastro!` });
            });
        };
    }


    /**
     * Lista todos os usuarios
     * @returns mensagem referente ao resultado obtido
     */
    listProdutosSimples() {
        return function (req, res) {
            const { id } = req.params;
            const id_user = req.query.id_user


            produtos.listProdutosSimples(id, id_user).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    /**
     * Lista todos os usuarios
     * @returns mensagem referente ao resultado obtido
     */
    listProdutosCompletos() {
        return function (req, res) {
            const { id } = req.params;
            const id_user = req.query.id_user


            produtos.listProdutosCompletos(id, id_user).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    testeFotos() {
        return function (req, res) {
            const { id } = req.params;


            produtos.testeFotos(id).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    /**
     * Lista todos os usuarios
     * @returns mensagem referente ao resultado obtido
     */
    listProdutosDeslogados() {
        return function (req, res) {
            const { id } = req.params;


            produtos.listProdutosDeslogado(id).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    /**
     * Lista todos os usuarios
     * @returns mensagem referente ao resultado obtido
     */
       listProdutosInvisiveis() {
        return function (req, res) {
            const { id } = req.params;

            produtos.listProdutosInvisiveis(id).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    /**
     * Lista todos os usuarios
     * @returns mensagem referente ao resultado obtido
     */
    listTodosProdutosAdmin() {
        return function (req, res) {
            const { id } = req.params;


            produtos.listTodosProdutosAdmin(id).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    /**
     * Lista todos os usuarios
     * @returns mensagem referente ao resultado obtido
     */
    listProdutosDescontosByUser() {
        return function (req, res) {
            const id_user = req.query.id_user;


            produtos.listProdutosDescontosByUser(id_user).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    /**
     * Lista todos os produtos com preco um
     * @returns mensagem referente ao resultado obtido
     */
    listProdutosPreco1() {
        return function (req, res) {
            const { id } = req.params;


            produtos.listProdutosPreco1(id).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    /**
     * Lista todos os produtos com preco 2
     * @returns mensagem referente ao resultado obtido
     */
    listProdutosPreco2() {
        return function (req, res) {
            const { id } = req.params;


            produtos.listProdutosPreco2(id).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    /**
     * Lista todos os produtos com preco 2
     * @returns mensagem referente ao resultado obtido
     */
    listProdutosPreco3() {
        return function (req, res) {
            const { id } = req.params;


            produtos.listProdutosPreco3(id).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    /**
     * Lista todos produtos e seus descontos
     * @returns mensagem referente ao resultado obtido
     */
    listProdutosDescontos() {
        return function (req, res) {
            const { id } = req.params;


            produtos.listProdutosDescontos(id).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    /**
     * Lista todos produtos e seus descontos
     * @returns mensagem referente ao resultado obtido
     */
    listProdutosTopAs() {
        return function (req, res) {
            const id_user = req.query.id_user;


            produtos.listProdutosTopAs(id_user).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    /**
 * Lista todos produtos e seus descontos
 * @returns mensagem referente ao resultado obtido
 */
    listProdutosByUserTipo() {
        return function (req, res) {
            const id_user = req.query.id_user;
            const tipo = req.query.tipo;


            produtos.listProdutosByUserTipo(id_user, tipo).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

        /**
 * Lista todos produtos e seus descontos
 * @returns mensagem referente ao resultado obtido
 */
        listProdutosByUserId() {
            return function (req, res) {
                
                const tipo = req.query.tipo;
                const id_user = req.query.id_user;
                const categoria = req.query.categoria;
                
    
    
                produtos.listProdutosByUserId(tipo, id_user, categoria).then((response) => {
                    if (response.error) res.status(response.code).send({ error: response.error });
                    else return res.status(200).send(response);
                }).catch((error) => {
                    throw new Error(error);
                });
            }
        }

    /**
    * Lista todos produtos visivel
    * @returns mensagem referente ao resultado obtido
    */
    listProdutosVisiveis() {
        return function (req, res) {
            const contrato = req.query.contrato


            produtos.listProdutosVisiveis(contrato).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    /**
* Lista todos produtos e seus descontos
* @returns mensagem referente ao resultado obtido
*/
    listProdutosByUserTipo2() {
        return function (req, res) {
            const tipo = req.query.tipo;


            produtos.listProdutosByUserTipo2(tipo).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }
    

    listTabelaDePrecoUser() {
        return function (req, res) {
            const id_user = req.query.id_user;


            produtos.listTabelaDePrecoUser(id_user).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    /**
     * Lista todos produtos e seus descontos
     * @returns mensagem referente ao resultado obtido
     */
    listProdutosDistribuidor() {
        return function (req, res) {
            const id_user = req.query.id_user;


            produtos.listProdutosDistribuidor(id_user).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    listLinhaProdutos() {
        return function (req, res) {


            produtos.listLinhaProdutos().then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                console.error(error);
                throw new Error(error);
            });
        }
    }

    /**
     * Lista todos produtos e seus descontos
     * @returns mensagem referente ao resultado obtido
     */
    listProdutosCategoria() {
        return function (req, res) {
            const num = req.query.num;


            produtos.listProdutosCategoria(num).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    /**
     * Delete um usuário
     * @returns mensagem referente ao resultado obtido
     */
    delete() {
        return function (req, res) {
            const { id } = req.params;

            produtos.delete(id).then((response) => {
                if (response.error) res.status(response.code).send(response);
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }


    /**
     * Atualiza os dados de um usuário
     * @returns mensagem referente ao resultado obtido
     */
    updatePreco1() {
        return function (req, res) {
            const data = req.body;

            produtos.updatePreco1(data).then((response) => {
                if (response.error) res.status(400).send(response);
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    /**
     * Atualiza os dados de um usuário
     * @returns mensagem referente ao resultado obtido
     */
    updatePreco2() {
        return function (req, res) {
            const data = req.body;

            produtos.updatePreco2(data).then((response) => {
                if (response.error) res.status(400).send(response);
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    /**
 * Atualiza os dados de um usuário
 * @returns mensagem referente ao resultado obtido
 */
    updatePreco3() {
        return function (req, res) {
            const data = req.body;

            produtos.updatePreco3(data).then((response) => {
                if (response.error) res.status(400).send(response);
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    setVisivelExclusivo() {
        return function (req, res) {
            const data = req.body;

            produtos.setVisivelExclusivo(data).then((response) => {
                if (response.error) res.status(400).send(response);
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    setExcluido() {
        return function (req, res) {
            const data = req.body;

            produtos.setExcluido(data).then((response) => {
                if (response.error) res.status(400).send(response);
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }
        /**
     * Atualiza os dados de um usuário
     * @returns mensagem referente ao resultado obtido
     */
        updateProduto() {
            return function (req, res) {
                const data = req.body;
                const file = req.file

                produtos.updateProduto(data, file).then((response) => {
                    if (response.error) res.status(400).send(response);
                    else return res.status(200).send(response);
                }).catch((error) => {
                    console.error(error);
                    throw new Error(error);
                });
            }
        }

}

module.exports = produtosController;