const Vendedores = require("../infra/repository/vendedores.repository");
const vendedores = new Vendedores();

const jwt = require("jsonwebtoken");

class vendedoresController {


  /**
   * Registra um vendedor
   * @returns mensagem referente ao resultado obtido
   */
  register() {
    return function (req, res) {
      const data = req.body;

      vendedores
        .register(data)
        .then((response) => {
          if (response.error)
            return res.status(response.code).send({ error: response.error });
          else return res.status(200).json(response);
        })
        .catch((error) => {
          console.log(error);
          
          return res
            .status(400)
            .send({ error: `Não foi possível realizar o cadastro!` });
        });
    };
  }

  /**
   * Lista todos os vendedores
   * @returns mensagem referente ao resultado obtido
   */
  list() {
    return function (req, res) {
      const { id } = req.params;

      vendedores
        .list(id)
        .then((response) => {
          if (response.error)
            res.status(response.code).send({ error: response.error });
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }
  /**
   * Delete um vendedor
   * @returns mensagem referente ao resultado obtido
   */
  delete() {
    return function (req, res) {
      const { id } = req.params;

      vendedores
        .delete(id)
        .then((response) => {
          if (response.error) res.status(response.code).send(response);
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }
  /**
   * Atualiza os dados de um vendedor
   * @returns mensagem referente ao resultado obtido
   */
  update() {
    return function (req, res) {
      const data = req.body;

      vendedores
        .update(data)
        .then((response) => {
          if (response.error) res.status(response.code).send(response);
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  /**
   * Realiza o login de um vendedor
   * @returns mensagem referente ao resultado obtido
   */
    login() {
      return function (req, res) {
        const data = req.body;

        vendedores
          .login(data)
          .then((response) => {
            if (response.error)
              return res.status(response.code).send({ error: response.error });
            else return res.status(200).json(response);
          })
          .catch((error) => {
            console.log(error);
            return res
              .status(400)
              .send({ error: `Não foi possivel realizar o login` });
          });
      };
    }


    listQtdeTotalVendedoresContrato() {
      return function (req, res) {
        const dataInicio = req.query.dataInicio;
        const dataFim = req.query.dataFim;
  
        vendedores
          .listQtdeTotalVendedoresContrato(dataInicio, dataFim)
          .then((response) => {
            if (response.error)
              res.status(response.code).send({ error: response.error });
            else return res.status(200).send(response);
          })
          .catch((error) => {
            throw new Error(error);
          });
      };
    }



    listClientesDoVendedor() {
      return function (req, res) {
        const id_vendedor = req.query.id_vendedor;
  
        vendedores
          .listClientesDoVendedor(id_vendedor)
          .then((response) => {
            if (response.error)
              res.status(response.code).send({ error: response.error });
            else return res.status(200).send(response);
          })
          .catch((error) => {
            throw new Error(error);
          });
      };
    }

    listQtdeVendidaPorVendedor() {
      return function (req, res) {
        const id_vendedor = req.query.id_vendedor;

        const data_inicio1 = req.query.data_inicio1;
        const data_fim1= req.query.data_fim1;

        const data_inicio2 = req.query.data_inicio2;
        const data_fim2 = req.query.data_fim2;
  
        vendedores
          .listQtdeVendidaPorVendedor(id_vendedor, data_inicio1, data_fim1, data_inicio2, data_fim2)
          .then((response) => {
            if (response.error)
              res.status(response.code).send({ error: response.error });
            else return res.status(200).send(response);
          })
          .catch((error) => {
            throw new Error(error);
          });
      };
    }


    listTopProdutos() {
      return function (req, res) {
        const id_user = req.query.id_user;
        const periodo = req.query.periodo;

        vendedores
          .listTopProdutos(id_user, periodo)
          .then((response) => {
            if (response.error)
              res.status(response.code).send({ error: response.error });
            else return res.status(200).send(response);
          })
          .catch((error) => {
            throw new Error(error);
          });
      };
    }

    listTopProdutosVendedor() {
      return function (req, res) {
        const id_vendedor = req.query.id_vendedor;
        const periodo = req.query.periodo;

  
        vendedores
          .listTopProdutosVendedor(id_vendedor, periodo)
          .then((response) => {
            if (response.error)
              res.status(response.code).send({ error: response.error });
            else return res.status(200).send(response);
          })
          .catch((error) => {
            throw new Error(error);
          });
      };
    }


    listFormVendedorByPeriodo() {
      return function (req, res) {
        const id_vendedor = req.query.id_vendedor;
        const periodo = req.query.periodo;

  
        vendedores
          .listFormVendedorByPeriodo(id_vendedor, periodo)
          .then((response) => {
            if (response.error)
              res.status(response.code).send({ error: response.error });
            else return res.status(200).send(response);
          })
          .catch((error) => {
            throw new Error(error);
          });
      };
    }

    listDashboard() {
      return function (req, res) {
        const data_inicio1 = req.query.data_inicio1;
        const data_fim1 = req.query.data_fim1;

        const data_inicio2 = req.query.data_inicio2;
        const data_fim2 = req.query.data_fim2;

  
        vendedores
          .listDashboard(data_inicio1, data_fim1, data_inicio2, data_fim2)
          .then((response) => {
            if (response.error)
              res.status(response.code).send({ error: response.error });
            else return res.status(200).send(response);
          })
          .catch((error) => {
            console.error(error);
            throw new Error(error);
          });
      };
    }

    listFormUsuarioProdutos() {
      return function (req, res) {
        const periodo = req.query.periodo
        const id_user = req.query.id_user;

        const data_inicio1 = req.query.data_inicio1;
        const data_fim1= req.query.data_fim1;

        const data_inicio2 = req.query.data_inicio2;
        const data_fim2 = req.query.data_fim2;



  
        vendedores
          .listFormUsuarioProdutos(periodo, data_inicio1, data_fim1, data_inicio2, data_fim2, id_user)
          .then((response) => {
            if (response.error)
              res.status(response.code).send({ error: response.error });
            else return res.status(200).send(response);
          })
          .catch((error) => {
            throw new Error(error);
          });
      };
    }

    loginUser() {
      return function (req, res) {
        const data = req.body;
        const documento = req.query.documento;
  
      
        vendedores
          .loginUser(data, documento)
          .then((response) => {
            if (response.error)
              return res.status(response.code).send({ error: response.error });
            else return res.status(200).json(response);
          })
          .catch((error) => {
            console.log(error);
            return res
              .status(400)
              .send({ error: `Não foi possivel realizar o login` });
          });
      };
    }

    loginVendedor() {
      return function (req, res) {
        const data = req.body
        const documento = req.query.documento;

  
        vendedores
          .loginVendedor(data, documento)
          .then((response) => {
            if (response.error)
              res.status(response.code).send({ error: response.error });
            else return res.status(200).send(response);
          })
          .catch((error) => {
            throw new Error(error);
          });
      };
    }

    registerUserPorVendedor() {
      return function (req, res) {
        const data = req.body
        const file = req.file;

  
        vendedores
          .registerUserPorVendedor(data, file)
          .then((response) => {
            if (response.error)
              res.status(response.code).send({ error: response.error });
            else return res.status(200).send(response);
          })
          .catch((error) => {
            throw new Error(error);
          });
      };
    }


    listCountAllVendedorHTTP() {
      return function (req, res) {
        const id_vendedor = req.query.id_vendedor;
        const periodo = req.query.periodo;

  
        vendedores
          .listCountAllVendedorHTTP(id_vendedor, periodo)
          .then((response) => {
            if (response.error)
              res.status(response.code).send({ error: response.error });
            else return res.status(200).send(response);
          })
          .catch((error) => {
            throw new Error(error);
          });
      };
    }

    listCountAllVendedor(id_vendedor, callback) {
      return new Promise(async (resolve, reject) => {

      if (!db) return resolve({ error: "Banco de dados não configurado.", code: 203 });
      
  
      db.query(
          `SELECT 
    (SELECT COUNT(*) 
     FROM user 
     WHERE user.id_vendedor = ?) as clientes_vinculados,

    (SELECT SUM(pedidos.concluido) 
     FROM pedidos, user 
     WHERE 
         pedidos.id_user = user.id AND
         user.id_vendedor = ? AND
         pedidos.concluido = true 
         ) AS pedidos_fechados,

    (SELECT SUM(pedidos.quantidade) AS qtde_caixa
     FROM pedidos, user 
     WHERE 
         pedidos.id_user = user.id AND
         pedidos.concluido = true AND
         user.id_vendedor = ? 
         ) as qtde_caixa,

    (SELECT AVG(mensal.pedidos_por_mes) 
     FROM (
         SELECT COUNT(*) AS pedidos_por_mes 
         FROM pedidos
         INNER JOIN user ON pedidos.id_user = user.id
         WHERE 
             pedidos.concluido = true AND 
             user.id_vendedor = ? 
         GROUP BY EXTRACT(YEAR FROM pedidos.data), EXTRACT(MONTH FROM pedidos.data)
     ) AS mensal) AS media`,
          [id_vendedor, id_vendedor, id_vendedor, id_vendedor],
          (error, result) => {
              if (error) {
                  console.error(error);
                  return resolve({ error: "Erro ao consultar a quantidade total de pedidos.", code: 500 });
              }
  
  
              return resolve(result[0]);
          }
      );
  }
)}


/**
 * Consulta em tempo real quantidade de clientes cadastrados, pedidos concluidos, caixas vendidas no mes atual
 * E a meta de um vendedor  
 * @param {Int} id_vendedor id o vendedor a ser consultado
 * @param {*} callback 
 * @returns lista com as quntidades e metas mensais de um vendedor
 */
  listMetasMensal(id_vendedor, callback) {
    return new Promise(async (resolve, reject) => {

    if (!db) return resolve({ error: "Banco de dados não configurado.", code: 203 });


    db.query(
        `SELECT 
          ( 
            SELECT COUNT(*) AS qtde_cliente_mensal
            FROM user
            WHERE
            user.id_vendedor = ? AND
			      user.data_criado >= DATE_SUB(CURDATE(), INTERVAL DAYOFMONTH(CURDATE()) - 1 DAY) AND
            user.data_criado < DATE_ADD(DATE_SUB(CURDATE(), INTERVAL DAYOFMONTH(CURDATE()) - 1 DAY), INTERVAL 1 MONTH)
          ) AS qtde_cliente_mensal,
          (
            SELECT SUM(pedidos.quantidade) AS qtde_caixa
            FROM pedidos, user
            WHERE
            pedidos.id_user = user.id AND
            pedidos.concluido = true AND
            user.id_vendedor = ? AND
            pedidos.data >= DATE_SUB(CURDATE(), INTERVAL DAYOFMONTH(CURDATE()) - 1 DAY) AND
            pedidos.data < DATE_ADD(DATE_SUB(CURDATE(), INTERVAL DAYOFMONTH(CURDATE()) - 1 DAY), INTERVAL 1 MONTH)
          ) AS qtde_caixa_mensal,
          (
            SELECT SUM(pedidos.concluido) 
            FROM pedidos, user
            WHERE 
            pedidos.id_user = user.id AND
            user.id_vendedor = ? AND
            pedidos.concluido = true AND
            pedidos.data >= DATE_SUB(CURDATE(), INTERVAL DAYOFMONTH(CURDATE()) - 1 DAY) AND
            pedidos.data < DATE_ADD(DATE_SUB(CURDATE(), INTERVAL DAYOFMONTH(CURDATE()) - 1 DAY), INTERVAL 1 MONTH)
          ) AS qtde_pedidos_mensal,
          (
            SELECT vendedores.meta_caixas_mensais
            FROM vendedores
            WHERE
            vendedores.id = ?
          ) AS meta_caixas_mensais,
          (
            SELECT vendedores.meta_pedidos_mensais
            FROM vendedores
            WHERE
            vendedores.id = ?
          ) AS meta_pedidos_mensais,
          (
            SELECT vendedores.meta_clientes_mensais
            FROM vendedores
            WHERE
            vendedores.id = ?
          ) AS meta_clientes_mensais
           `,
        [id_vendedor, id_vendedor, id_vendedor, id_vendedor, id_vendedor , id_vendedor],
        (error, result) => {
            if (error) {
                console.error(error);
                return resolve({ error: "Erro ao consultar a quantidade e metas mensais.", code: 500 });
            }
            return resolve(result[0]);
        }
    );
  }
)}


/**
 * Consula todos os ultimos pedidos dos clientes vinculado do id_vendedor fornecido
 * @param {int} id_vendedor id do vendedor
 * @returns Todos os pedidos dos clientes do vendedor fornecido
 */
listUltimosPedidos(id_vendedor, callback) {
  return new Promise(async (resolve, reject) => {

  if (!db) return callback({ error: "Banco de dados não configurado." });

  // let query = ``;
  // switch (periodo) {
  //   case 'semanal':
  //     query = `YEARWEEK(pedidos.data, 1) = YEARWEEK(CURDATE(), 1) AND`
  //       console.log('Ação para semanal');
  //       break;
  //   case 'mensal':
  //       query = `YEAR(pedidos.data) = YEAR(CURDATE()) AND MONTH(pedidos.data) = MONTH(CURDATE()) AND`
  //       console.log('Ação para mensal');
  //       break;
  //   case 'anual':
  //     query = `YEAR(data) = YEAR(CURDATE()) AND`    
  //       console.log('Ação para anual');
  //       break;
  //   default:
  //     query = ``
  //       console.log('Ação para outro caso');
  // }


  db.query(
        `SELECT pedidos.id, user.id AS id_user, user.razao_social AS cliente, pedidos.quantidade, DATE_FORMAT(pedidos.data, "%d/%m/%Y") AS data, pedidos.hora
            FROM user, pedidos, vendedores
            WHERE
            user.id_vendedor = vendedores.id AND
            pedidos.id_user = user.id AND
            pedidos.concluido = true AND
            user.id_vendedor = ?
            ORDER BY pedidos.data DESC, pedidos.hora DESC`,
      [id_vendedor],
      (error, result) => {
          if (error) {
              console.error(error);
              return callback({ error: "Erro ao consultar a ultimos pedidos." });
          }
          return resolve(result);
      }
  );
  }
)}


listTopClientesVendedor() {
  return function (req, res) {
    const id_vendedor = req.query.id_vendedor;
    const periodo = req.query.periodo


    vendedores
      .listTopClientesVendedor(id_vendedor, periodo)
      .then((response) => {
        if (response.error)
          res.status(response.code).send({ error: response.error });
        else return res.status(200).send(response);
      })
      .catch((error) => {
        throw new Error(error);
      });
  };
}




// setupUltimosPedidos(socket) {
//   socket.on('listUltimosPedidos', (data) => {
//       const { id_vendedor } = data;

//       if (!id_vendedor) {
//           socket.emit('listUltimosPedidosResponse', { error: "ID do vendedor é obrigatório." });
//           return;
//       }

//       this.listUltimosPedidos(id_vendedor, (error, response) => {
//           if (error) {
//             console.error(error);
            
//               socket.emit('listUltimosPedidosResponse', { error: error });
//           } else {
//               socket.emit('listUltimosPedidosResponse', response);
//           }
//       });
//   });
// }


listVendasMensais() {
  return function (req, res) {
    const id_vendedor = req.query.id_vendedor;


    vendedores
      .listVendasMensais(id_vendedor)
      .then((response) => {
        if (response.error)
          res.status(response.code).send({ error: response.error });
        else return res.status(200).send(response);
      })
      .catch((error) => {
        throw new Error(error);
      });
  };
}

uploader() {
  return function (req, res) {
    const file = req.body.file;
    const data = req.body;

    vendedores.uploadFileToS3(data, file).then((response) => {
      if (response.error)
        res.status(response.code).send({ error: response.error });
      else return res.status(200).send(response);
    })
    .catch((error) => {
      console.error(error);
      
      throw new Error(error);
    });
}
}


getFileFromS3() {
  return function (req, res) {
    // const file = req.file;
    const id = req.query.id;

    vendedores.getFileFromS3(id).then((response) => {
      if (response.error)
        res.status(response.code).send({ error: response.error });
      else return res.status(200).send(response);
    })
    .catch((error) => {
      throw new Error(error);
    });
}
}
 


// uploader() {
//   return function (req, res) {
//     const file = req.file;

//     vendedores.uploader(file).then((response) => {
//       if (response.error) {
//         if (response.error instanceof uploadService.multer.MulterError) {
//           return res.status(400).send({ error: response.message });
//         }
//         return res.status(500).send({ error: response.message });
//       }
//         return res
//           .status(400)
//           .send({ error: `Não foi possível realizar o cadastro!` });
//     });
//   };
// }
//       .register(data)
//       .then((response) => {
//         if (response.error)
//           return res.status(response.code).send({ error: response.error });
//         else return res.status(200).json(response);
//       })
//       .catch((error) => {
//         console.log(error);
        
//         return res
//           .status(400)
//           .send({ error: `Não foi possível realizar o cadastro!` });
//       });
//   };
// }


listMetasMensalHTTP() {
  return function (req, res) {
    const id_vendedor = req.query.id_vendedor;
    const periodo = req.query.periodo;


    vendedores
      .listMetasMensalHTTP(id_vendedor, periodo)
      .then((response) => {
        if (response.error)
          res.status(response.code).send({ error: response.error });
        else return res.status(200).send(response);
      })
      .catch((error) => {
        throw new Error(error);
      });
  };
}

listUser() {
  return function (req, res) {
    const id_user = req.query.id_user;
    const id_vendedor = req.query.id_vendedor;



    vendedores
      .listUser(id_user, id_vendedor)
      .then((response) => {
        if (response.error)
          res.status(response.code).send({ error: response.error });
        else return res.status(200).send(response);
      })
      .catch((error) => {
        throw new Error(error);
      });
  };
}

edit() {
  return function (req, res) {
    const data = req.body;

    vendedores
      .edit(data)
      .then((response) => {
        if (response.error) res.status(400).send(response);
        else return res.status(200).send(response);
      })
      .catch((error) => {
        throw new Error(error);
      });
  };
}

listDescontosByUserVendedor() {
  return function (req, res) {
    // const data = req.body;
    const id_user = req.query.id_user

    vendedores
      .listDescontosByUserVendedor(id_user)
      .then((response) => {
        if (response.error) res.status(400).send(response);
        else return res.status(200).send(response);
      })
      .catch((error) => {
        throw new Error(error);
      });
  };
}

updateProduto() {
  return function (req, res) {
      const data = req.body;
      const id = req.query.id
      const id_vendedor = req.query.id_vendedor;

      vendedores.updateProduto(data, id, id_vendedor).then((response) => {
          if (response.error) res.status(400).send(response);
          else return res.status(200).send(response);
      }).catch((error) => {
          throw new Error(error);
      });
  }
}


  /**
   * Envia um email para recuperação de senha
   * @returns mensagem referente ao resultado obtido
   */
  recover() {
    return function (req, res) {
      const data = req.body;
      vendedores
        .recover(data)
        .then((response) => {
          if (response.error)
            return res.status(response.code).send({ error: response.error });
          else return res.status(200).json(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

    /**
   * Envia um email para recuperação de senha
   * @returns mensagem referente ao resultado obtido
   */
    emailToUser() {
      return function (req, res) {
        const data = req.body;
        vendedores
          .emailToUser(data)
          .then((response) => {
            if (response.error)
              return res.status(response.code).send({ error: response.error });
            else return res.status(200).json(response);
          })
          .catch((error) => {
            throw new Error(error);
          });
      };
  }

  
  registerDesconto() {
    return function (req, res) {
        const  data  = req.body;

        vendedores.registerDesconto(data).then((response) => {
            if (response.error) return res.status(response.code).send({ error: response.error });
            else return res.status(200).json(response);
        }).catch((error) => {
            return res.status(400).send({ error: `Não foi possível realizar o cadastro!` });
        });
    };
}


}

module.exports = vendedoresController;

