const express = require("express");
const routes = express.Router();
const auth = require("../config/auth/auth");
const multer = require("multer");

const storage = multer.memoryStorage();


const upload = multer({ 
    storage,
    dest: "uploads/"
})

const PrecosController = require("../controller/precos.controller")
const precosController = new PrecosController();

const AdminController = require("../controller/admin.controller");
const adminController = new AdminController();

const PedidosController = require("../controller/pedidos.controller");
const pedidosController = new PedidosController();

const DescontosController = require("../controller/descontos.controller");
const descontosController = new DescontosController();

const ProdutosController = require("../controller/produtos.controller");
const produtosController = new ProdutosController();

const ProdutosDisponiveisController = require("../controller/produtos.disponiveis.controller")
const produtosDisponiveisController = new ProdutosDisponiveisController();

const MensagensController = require("../controller/mensagens.controller");
const mensagensController = new MensagensController();

const UsersController = require("../controller/usuarios.controller");
const usersController = new UsersController();

const VendedorController = require("../controller/vendedores.controller");
const vendedorController = new VendedorController();

const FuncionariosController = require("../controller/funcionario.controller")
const funcionariosController = new FuncionariosController();

const MetasController = require("../controller/metas.controller")
const metasController = new MetasController();

const LogsController = require("../controller/logs.controller");
const logsController = new LogsController();

routes.get("/precos", precosController.list());
routes.get("/produtos", precosController.listProdutosComPrecos());
routes.get("/tabelas", precosController.listTodosProdutosPorTabela());
routes.delete("/tabelas", precosController.delete());
routes.patch("/tabelas", precosController.updatePreco());
routes.get("/tabela", precosController.listPrecosByUserTipo());
routes.get("/nomeTabelas", precosController.listTabelaPrecoNome());



routes.post("/precos", precosController.register());
routes.patch("/precos", precosController.updateTabelaPreco());

routes.get("/topProdutos", vendedorController.listTopProdutos());
routes.get("/relatorio", vendedorController.listFormUsuarioProdutos());
routes.get("/relatorio_vendedores", vendedorController.listQtdeVendidaPorVendedor());
routes.get("/dashboard_vendedor", vendedorController.listDashboard());


routes.patch("/aprovar", adminController.aprovarUsuario());
routes.patch("/recusar", adminController.RecusarUsuario());
routes.get("/pendentes", adminController.listUserPendente());
routes.get("/aprovados", adminController.listUserAprovado());
routes.get("/recusados", adminController.listUserRecusado());
routes.get("/usuarios", adminController.listAllUser());
routes.get("/relatorioPedidos", adminController.listPedidosNaoEfetuados());
routes.patch("/ultra", adminController.updateUltra());
routes.get("/ultra", adminController.listUltra());
routes.patch("/mudarSenha", adminController.recover());
routes.get("/pedidosEndereco", adminController.listPedidosPorEndereco());
routes.get("/topVendedores", adminController.listTopVendedores());
routes.get("/topCidades", adminController.listTopCidades());

routes.get("/metas", metasController.list());
routes.post("/metas", metasController.register());
routes.patch("/metas", metasController.update());
routes.delete("/metas/:id", metasController.delete())

routes.get("/countAllUserType", adminController.countAllUserTypes());
routes.get("/historico", adminController.listUserPedidosConluido());
routes.get("/historicoUser", adminController.listHistoricoPedidoByUser());
routes.get("/userPedidos", adminController.listUserHistorico())

routes.get("/grafico", pedidosController.listForm());
routes.get("/graficoByPeriodo", pedidosController.listFormByPeriodo());
routes.get("/listaHistorico", pedidosController.list())

routes.get("/usuariosDiaPedido", adminController.listAllUserDiaPedido());

routes.get("/topAs", adminController.listUserAprovadoTopAS());
routes.get("/atacados", adminController.listUserAprovadoAtacado());
routes.get("/distribuidores", adminController.listUserAprovadoDistribuidor());


// routes.get("/produtos", produtosController.listTodosProdutosAdmin());;
routes.post("/produtos", upload.single("file"), produtosController.register());
routes.patch("/produtos", upload.single("file"), produtosController.updateProduto());
routes.patch("/preco1Produto", produtosController.updatePreco1());
routes.patch("/preco2Produto", produtosController.updatePreco2());
routes.patch("/preco3Produto", produtosController.updatePreco3());
routes.patch("/excluirProduto", produtosController.setExcluido());
routes.delete("/produtos", produtosController.delete());
routes.get("/linhas", produtosController.listLinhaProdutos());
routes.patch("/visivel", produtosController.setVisivelExclusivo());


routes.get("/produtosPreco1", produtosController.listProdutosPreco1());
routes.get("/produtosPreco2", produtosController.listProdutosPreco2());
routes.get("/produtosPreco3", produtosController.listProdutosPreco3());
routes.get("/produtosCategoria", produtosController.listProdutosCategoria());
routes.get("/produtosInvisiveis", produtosController.listProdutosInvisiveis());

routes.get("/produtosAtacado", produtosController.listProdutosByUserTipo());
routes.get("/produtosTopAs", produtosController.listProdutosTopAs());
routes.get("/produtosDistribuidores", produtosController.listProdutosDistribuidor());


routes.post("/vincularProdutoUser", produtosDisponiveisController.vincularProdutoUser())
routes.delete("/desvincularProdutoUser", produtosDisponiveisController.desvincularProdutoDeUsuario());
routes.get("/produtosVinculados", produtosDisponiveisController.listProdutoDisponiveisByUser())
routes.get("/usuariosVinculados", produtosDisponiveisController.listProdutoComVinculos());


routes.get("/edit", adminController.listEdit());
routes.patch("/edit", adminController.edit());
routes.patch("/updateFullUser", adminController.updateFullUser());
routes.patch("/desvincularVendedor", adminController.desvincularVendedor());


routes.get("/produtosDescontos", produtosController.listProdutosDescontos());


routes.post("/descontos", descontosController.register());
routes.get("/descontos", descontosController.list());
routes.delete("/descontos/:id", descontosController.delete());
routes.patch("/descontos", descontosController.update());
routes.get("/descontosUser", descontosController.listDescontosByUser());

routes.get("/lembrete", mensagensController.listAssuntoMensagens());
routes.get("/mensagensVinculadas", mensagensController.listMensagensVinculadas());
routes.patch("/mensagens", upload.single("file"), mensagensController.update());
routes.delete("/mensagens/:id", mensagensController.delete());
routes.post("/mensagens", upload.single("file"), mensagensController.register());
routes.post("/vincularMensagem", mensagensController.vincularUsuarioMensangem());
routes.delete("/desvincularMensagem", mensagensController.desvincularMensagem())
routes.get("/mensagens", mensagensController.listMensagens());
routes.post("/enviarLembrete", mensagensController.enviarLembreteParaUsuario());

routes.get("/logs", logsController.listlogsByUser());
routes.delete("/logs/:id", logsController.delete());

routes.patch("/aprovarFunc", adminController.aprovarFuncionario());
routes.patch("/recusarFunc", adminController.recusarFuncionario());

routes.get("/funcionarios", funcionariosController.listAllFuncs());


routes.get("/usuario", usersController.list());
routes.post("/login", adminController.login());



module.exports = routes;